;Always the first step: load the VIGRA CL Lib
(load "vigracl.cl")

(in-package :vigracl)

;Just as a simple example: My own gauss approx. filter for images
;Don't forget to compile -> enormeous speed-up!
(defun smooth_pixel (arr i j)
       (setf (aref arr i j) 
       	     (/ (+ (* 4 (aref arr i       j      ))
       	     	   (* 2 (aref arr (+ i 1) j      ))
	     	   (* 2 (aref arr (- i 1) j      ))
	     	   (* 2 (aref arr i       (+ j 1)))
	     	   (* 2 (aref arr i       (- j 1)))
       	     	   (* 1 (aref arr (+ i 1) (+ j 1)))
	     	   (* 1 (aref arr (+ i 1) (- j 1)))
	     	   (* 1 (aref arr (- i 1) (+ j 1)))
	     	   (* 1 (aref arr (- i 1) (- j 1))))
               16)))
(compile 'smooth_pixel)

(defun mysmooth-band (in_arr)
  (let ((arr (copy-array in_arr)))
      (do* ((j 1 (+ j 1)))
      	  ((= j (- (array-dimension arr 1) 1)) arr)
	      (do* ((i 1 (+ i 1)))
      	      	  ((= i (- (array-dimension arr 0) 1)))
		      (smooth_pixel arr i j)))))
(compile 'mysmooth-band)


;A second example: My own gauss approx. filter using convolution kernel
(setq gauss_kernel (make-array '(3 3)
                    :element-type '(double-float)
                    :initial-element 0.0625d+00))
(setf  (aref gauss_kernel 1 0) 0.125d+00)
(setf  (aref gauss_kernel 0 1) 0.125d+00)
(setf  (aref gauss_kernel 1 1) 0.25d+00)
(setf  (aref gauss_kernel 2 0) 0.125d+00)
(setf  (aref gauss_kernel 0 2) 0.125d+00)

(print "loading lenna-image")
(time (setq img (loadimage  "images/lenna_face.png")))

(print "performance test gaussian smoothing")
(print "vigra-implicit-method:")
(time (setq img_ig  (gsmooth img 0.5)))
(print "vigra-std.-convolution-method:")
(time (setq img_eg (convolveimage img gauss_kernel)))
(print "my-own-method:")
(time (setq img_mg (mapcar #'mysmooth-band img)))

					;Tensor tests
(showimage  (image->red img))
(showimage  (image->green img))
(showimage  (image->blue img))
(showimage  img)

; Tensor tests
(setq img1_st  (structuretensor img 1.0 4.0))
(showimage  (first img1_st))
(showimage  (second img1_st))
(showimage  (third img1_st))

; boundary tensor
(setq img1_bt  (boundarytensor img 1.0))
(showimage  (first img1_bt) )
(showimage  (second img1_bt))
(showimage  (third img1_bt))

; boundary tensor without 0 order parts
(setq img1_bt1  (boundarytensor1 img 1.0))
(showimage  (first img1_bt1))
(showimage  (second img1_bt1) )
(showimage  (third img1_bt1))

;;tensor to eigen repr.
(setq img1_st_te (tensoreigenrepresentation img1_st))
(showimage  (first img1_st_te))
(showimage  (second img1_st_te))
(showimage  (third img1_st_te))

;tensor trace                
(setq img1_st_tt (tensortrace img1_st))
(showimage  img1_st_tt )

;tensor to edge corner
(setq img1_st_ec (tensortoedgecorner img1_st))
(showimage (first img1_st_ec) )
(showimage (second img1_st_ec))
(showimage (third img1_st_ec) )

;tensor to hourglass-filtered tensor
(setq img1_st_hg (hourglassfilter img1_st 1.0 1.0))
(showimage (first img1_st_hg))
(showimage (second img1_st_hg))
(showimage (third img1_st_hg))



;FFT Tests following
(time (setq imgrect (loadimage  "images/rect.gif")))

;do fast fourier transform
(print "performing fft on image")
(time (setq imgrectfft (fouriertransform imgrect)))

;Calculate the amplitude of the frequency spectrum
(defun complex-magnitude (real imag)
  (sqrt (+ (* real real) (* imag imag))))

(setq imgfft (image-map #'complex-magnitude (first imgrectfft) (second imgrectfft)))

; Save:
;real part
(print "saving resulting fft images")
(saveimage (first imgrectfft) "images/rect-fft-real.png")
;imaginary part
(saveimage (second imgrectfft) "images/rect-fft-imag.png")
;amplitude
(saveimage imgfft "images/rect-fft.png")
;amplitude (sqrt scaled)
(saveimage (image-map #'sqrt imgfft) "images/rect-fft-sqrt.png")
;amplitude (log scaled)
(saveimage (image-map #'log imgfft) "images/rect-fft-log.png")

;Testing vigra methods
(print "performing watershed transform on resized gradient image")
(time (setq img2  (regionimagetocrackedgeimage(labelimage
	    (watersheds
	     (ggradient 
	      (resizeimage img 800 800 4)
	      3.0))) 0.0)))


(print "testing rotation and reflection functions on image")
(time (setq img4 (reflectimage img 3)))
(time (setq img5 (rotateimage img 15.0 3)))


(setq matrix (make-array '(3 3)
                    :element-type '(double-float)
                    :initial-element 0.0d+00))
(setq theta (/(* -15d+00 3.14157d+00) 180d+00))
(setf  (aref matrix 0 0) (cos theta))
(setf  (aref matrix 1 1) (cos theta))
(setf  (aref matrix 0 1)  (* -1.0 (sin theta)))
(setf  (aref matrix 1 0)  (sin theta))
(setf  (aref matrix 2 2)  1.0d+00)

(time (setq img5aff (affinewarpimage img matrix 3)))

(print "performing distance transform on canny edges of image")
(time (setq img6 (distancetransform
		  (cannyedgeimage img 1.8 0.1 100.0)
		  0.0 2)))

(print "testing difference of exponential edge detection on image")
(time (setq img7 (differenceofexponentialedgeimage img 0.8 0.1 100.0)))

(print "testing nearly all filters")
(time (setq img8 (gsmooth img 3.0)))
(time (setq img9 (laplacianofgaussian img 3.0)))
(time (setq img10 (gsharpening img 0.5 3.0)))
(time (setq img11 (sharpening img 3.0)))
(time (setq img12 (nonlineardiffusion img 0.1 2.0)))

(print "saving resulting images")
(saveimage img2  "images/lenna-relabeled-watersheds-on-resized-gradient-image.png")

(saveimage img4  "images/lenna-reflected-both.png")
(saveimage img5  "images/lenna-rotated-15deg.png")
(saveimage img5aff  "images/lenna-rotated-15deg-aff.png")
(saveimage img6  "images/lenna-disttransform-on-canny.png")
(saveimage img7  "images/lenna-diff_of_exp.png")
(saveimage img8  "images/lenna-gsmooth-3.0.png")
(saveimage img9  "images/lenna-log-3.0.png")
(saveimage img10 "images/lenna-gsharpening-0.5-3.0.png")
(saveimage img11 "images/lenna-sharpening-3.0.png")
(saveimage img12 "images/lenna-nonlineardiffusion-0.1-2.0.png")