;###############################################################################
;###################          Generic Convolution           ####################
(ff:def-foreign-call vigra_convolveimage_c 
    ((arr      (:array :float))
     (arr2      (:array :float))
     (kernel      (:array :double))
     (width    :int fixnum) 
     (height   :int fixnum)
     (kernel_width    :int fixnum) 
     (kernel_height   :int fixnum))
  :returning  (:int fixnum))

(defun convolveimage-band (arr kernel_matrix)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (kernel_width  (array-dimension kernel_matrix 0))
	 (kernel_height (array-dimension kernel_matrix 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_convolveimage_c arr arr2 kernel_matrix width height  kernel_width kernel_height)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.colvolveimage: Convolution with kernel failed!")
      ((2) "Error in vigracl.filters.colvolveimage: Kernel dimensions must be odd!"))))

(defun convolveimage (image kernel_matrix)
  (mapcar #'(lambda (arr) (convolveimage-band arr kernel_matrix)) image))


;###############################################################################
;###################        Separable Convolution           ####################
(ff:def-foreign-call vigra_separableconvolveimage_c 
    ((arr      (:array :float))
     (arr2      (:array :float))
     (kernel_h      (:array :double))
     (kernel_v      (:array :double))
     (width    :int fixnum) 
     (height   :int fixnum)
     (kernel_width    :int fixnum) 
     (kernel_height   :int fixnum))
  :returning  (:int fixnum))

(defun separableconvolveimage-band (arr kernel_h kernel_v)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (kernel_width  (array-dimension kernel_h 0))
	 (kernel_height (array-dimension kernel_v 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_separableconvolveimage_c arr arr2 kernel_h  kernel_v width height  kernel_width kernel_height)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.separablecolvolveimage: Convolution with kernel failed!")
      ((2) "Error in vigracl.filters.separablecolvolveimage: Kernel dimensions must be odd!"))))

(defun separableconvolveimage (image  kernel_h kernel_v)
  (mapcar #'(lambda (arr) (separableconvolveimage-band arr kernel_h kernel_v)) image))


;###############################################################################
;###################          Gaussian Smoothing            ####################

(ff:def-foreign-call vigra_gaussiansmoothing_c 
    ((arr      (:array :float))
     (arr2      (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (sigma   :float))
  :returning  (:int fixnum))

(defun gsmooth-band (arr sigma)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0)))
    (case  (vigra_gaussiansmoothing_c arr arr2 width height sigma)
      ((0) arr2)
      ((1) "Error in vigracl.filters.gsmooth: Gaussian smoothing failed!"))))

(defun gsmooth (image sigma)
  (mapcar #'(lambda (arr) (gsmooth-band arr sigma)) image))

;###############################################################################
;###################      Gaussian Gradient (nX & nY)       ####################

(ff:def-foreign-call vigra_gaussiangradient_c
    ((arr      (:array :float))
     (arr_x     (:array :float))
     (arr_y     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (sigma   :float))
  :returning  (:int fixnum))


(defun gaussiangradient-band (arr sigma)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr_x (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (arr_y (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_gaussiangradient_c arr arr_x arr_y width height sigma)))
    (case foo
      ((0) (list arr_x arr_y))
      ((1) "Error in vigracl.filters.ggradient: Gaussian gradient magnitude failed!"))))

(defun gaussiangradient (image sigma)
  (pivot-list (mapcar #'(lambda (arr) (gaussiangradient-band arr sigma)) image)))

;###############################################################################
;###################      Gaussian Gradient (Magnitude)     ####################

(ff:def-foreign-call vigra_gaussiangradientmagnitude_c
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (sigma   :float))
  :returning  (:int fixnum))


(defun ggradient-band (arr sigma)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_gaussiangradientmagnitude_c arr arr2 width height sigma)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.ggradient: Gaussian gradient magnitude failed!"))))

(defun ggradient (image sigma)
  (mapcar #'(lambda (arr) (ggradient-band arr sigma)) image))


;###############################################################################
;###################        Laplacian Of Gaussian           ####################

(ff:def-foreign-call vigra_laplacianofgaussian_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (scale   :float))
  :returning  (:int fixnum))

(defun laplacianofgaussian-band (arr scale)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_laplacianofgaussian_c arr arr2 width height scale)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.laplacianofgaussian: Laplacian of Gaussian failed!"))))

(defun laplacianofgaussian (image scale)
  (mapcar #'(lambda (arr) (laplacianofgaussian-band arr scale)) image))

  

;###############################################################################
;#############    Hessian Matrix of 2. order deriv gaussians      ##############

(ff:def-foreign-call vigra_hessianmatrixofgaussian_c
    ((arr     (:array :float))
     (arr_xx  (:array :float))
     (arr_xy  (:array :float))
     (arr_yy  (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (scale   :float))
  :returning  (:int fixnum))

(defun hessianmatrixofgaussian-band (arr scale)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr_xx (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (arr_xy (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (arr_yy (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_hessianmatrixofgaussian_c arr arr_xx arr_xy arr_yy width height scale)))
    (case foo
      ((0) (list arr_xx arr_xy arr_yy))
      ((1) "Error in vigracl.filters.hessianmatrixofgaussian: Calculation of Hessian Matrix (of gaussian 2. deriv.) failed!"))))

(defun hessianmatrixofgaussian (image scale)
  (pivot-list (mapcar #'(lambda (arr)  (hessianmatrixofgaussian-band arr scale)) image)))

;###############################################################################
;#########  Structure Tensor using inner and outer gaussians      ##############

(ff:def-foreign-call vigra_structuretensor_c
    ((arr     (:array :float))
     (arr_xx  (:array :float))
     (arr_xy  (:array :float))
     (arr_yy  (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (inner_scale   :float)
     (outer_scale   :float))
  :returning  (:int fixnum))

(defun structuretensor-band (arr inner_scale outer_scale)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr_xx (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (arr_xy (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (arr_yy (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_structuretensor_c arr arr_xx arr_xy arr_yy width height inner_scale outer_scale)))
    (case foo
      ((0) (list arr_xx arr_xy arr_yy))
      ((1) "Error in vigracl.filters.structuretensor: Calculation of Structure Tensor failed!"))))

(defun structuretensor (image inner_scale outer_scale)
  (pivot-list (mapcar #'(lambda (arr)  (structuretensor-band arr inner_scale outer_scale)) image)))


;###############################################################################
;#############               Koethes Boundary tensor              ##############

(ff:def-foreign-call vigra_boundarytensor_c
    ((arr     (:array :float))
     (arr_xx  (:array :float))
     (arr_xy  (:array :float))
     (arr_yy  (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (scale   :float))
  :returning  (:int fixnum))

(defun boundarytensor-band (arr scale)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr_xx (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (arr_xy (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (arr_yy (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_boundarytensor_c arr arr_xx arr_xy arr_yy width height scale)))
    (case foo
      ((0) (list arr_xx arr_xy arr_yy))
      ((1) "Error in vigracl.filters.boundarytensor: Calculation of Koethes Boundary Tensor failed!"))))

(defun boundarytensor (image scale)
  (pivot-list (mapcar #'(lambda (arr)  (boundarytensor-band arr scale)) image)))

;###############################################################################
;#######     Koethes Boundary tensor (without 0th order response)      #########

(ff:def-foreign-call vigra_boundarytensor1_c
    ((arr     (:array :float))
     (arr_xx  (:array :float))
     (arr_xy  (:array :float))
     (arr_yy  (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (scale   :float))
  :returning  (:int fixnum))

(defun boundarytensor1-band (arr scale)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr_xx (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (arr_xy (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (arr_yy (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_boundarytensor1_c arr arr_xx arr_xy arr_yy width height scale)))
    (case foo
      ((0) (list arr_xx arr_xy arr_yy))
      ((1) "Error in vigracl.filters.boundarytensor1: Calculation of Koethes Boundary Tensor (without 0th order response) failed!"))))

(defun boundarytensor1 (image scale)
  (pivot-list (mapcar #'(lambda (arr)  (boundarytensor1-band arr scale)) image)))

;###############################################################################
;#########     Eigenvalue Representation of a Tensor              ##############

(ff:def-foreign-call vigra_tensoreigenrepresentation_c
    ((arr_xx    (:array :float))
     (arr_xy    (:array :float))
     (arr_yy    (:array :float))
     (arr_lEV   (:array :float))
     (arr_sEV   (:array :float))
     (arr_angle (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum))
  :returning  (:int fixnum))

(defun tensoreigenrepresentation-band (tensor)
  (let* ((arr_xx (first  tensor))
	 (arr_xy (second tensor))
	 (arr_yy (third  tensor))
	 (width  (array-dimension arr_xx 0))
	 (height (array-dimension arr_xx 1))
	 (arr_lEV   (make-array (array-dimensions arr_xx)
			   :element-type (array-element-type arr_xx)
			   :initial-element 0.0))
	 (arr_sEV   (make-array (array-dimensions arr_xx)
			   :element-type (array-element-type arr_xx)
			   :initial-element 0.0))
	 (arr_angle (make-array (array-dimensions arr_xx)
			   :element-type (array-element-type arr_xx)
			   :initial-element 0.0))
	 (foo   (vigra_tensoreigenrepresentation_c arr_xx arr_xy arr_yy arr_lEV arr_sEV arr_angle width height)))
    (case foo
      ((0) (list arr_lEV arr_sEV arr_angle))
      ((1) "Error in vigracl.filters.tensoreigenrepresentation: Calculation of the  Eigenvalue Representation of the Tensor failed!"))))

(defun tensoreigenrepresentation (tensor_image)
  (pivot-list (mapcar #'(lambda (tensor)  (tensoreigenrepresentation-band tensor)) (pivot-list tensor_image))))

;###############################################################################
;#########                    Trace of a Tensor                   ##############

(ff:def-foreign-call vigra_tensortrace_c
    ((arr_xx    (:array :float))
     (arr_xy    (:array :float))
     (arr_yy    (:array :float))
     (arr_trace   (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum))
  :returning  (:int fixnum))

(defun tensortrace-band (tensor)
  (let* ((arr_xx (first  tensor))
	 (arr_xy (second tensor))
	 (arr_yy (third  tensor))
	 (width  (array-dimension arr_xx 0))
	 (height (array-dimension arr_xx 1))
	 (arr_trace   (make-array (array-dimensions arr_xx)
			   :element-type (array-element-type arr_xx)
			   :initial-element 0.0))
	 (foo   (vigra_tensortrace_c arr_xx arr_xy arr_yy arr_trace width height)))
    (case foo
      ((0) arr_trace)
      ((1)  "Error in vigracl.filters.tensortrace: Calculation of the trace of the Tensor failed!"))))

(defun tensortrace (tensor_image)
  (mapcar #'(lambda (tensor)  (tensortrace-band tensor)) (pivot-list tensor_image)))

;###############################################################################
;#########     Edge/Corner Representation of a Tensor             ##############

(ff:def-foreign-call vigra_tensortoedgecorner_c
    ((arr_xx    (:array :float))
     (arr_xy    (:array :float))
     (arr_yy    (:array :float))
     (arr_edgeness   (:array :float))
     (arr_orientation   (:array :float))
     (arr_cornerness  (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum))
  :returning  (:int fixnum))

(defun tensortoedgecorner-band (tensor)
  (let* ((arr_xx (first  tensor))
	 (arr_xy (second tensor))
	 (arr_yy (third  tensor))
	 (width  (array-dimension arr_xx 0))
	 (height (array-dimension arr_xx 1))
	 (arr_edgeness   (make-array (array-dimensions arr_xx)
			   :element-type (array-element-type arr_xx)
			   :initial-element 0.0))
	 (arr_orientation   (make-array (array-dimensions arr_xx)
			   :element-type (array-element-type arr_xx)
			   :initial-element 0.0))
	 (arr_cornerness  (make-array (array-dimensions arr_xx)
			   :element-type (array-element-type arr_xx)
			   :initial-element 0.0))
	 (foo   (vigra_tensortoedgecorner_c arr_xx arr_xy arr_yy arr_edgeness arr_orientation arr_cornerness width height)))
    (case foo
      ((0) (list arr_edgeness arr_orientation arr_cornerness))
      ((1) "Error in vigracl.filters.tensortoedgecorner: Calculation of the Edge/Corner Representation of the Tensor failed!"))))

(defun tensortoedgecorner (tensor_image)
  (pivot-list (mapcar #'(lambda (tensor)  (tensortoedgecorner-band tensor)) (pivot-list tensor_image))))

;###############################################################################
;#########           Hourglass-Filtering  of a Tensor             ##############

(ff:def-foreign-call vigra_hourglassfilter_c
    ((arr_xx    (:array :float))
     (arr_xy    (:array :float))
     (arr_yy    (:array :float))
     (arr_hg_xx  (:array :float))
     (arr_hg_xy  (:array :float))
     (arr_hg_yy  (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (sigma   :float)
     (rho     :float))
  :returning  (:int fixnum))

(defun hourglassfilter-band (tensor sigma rho)
  (let* ((arr_xx (first  tensor))
	 (arr_xy (second tensor))
	 (arr_yy (third  tensor))
	 (width  (array-dimension arr_xx 0))
	 (height (array-dimension arr_xx 1))
	 (arr_hg_xx   (make-array (array-dimensions arr_xx)
			   :element-type (array-element-type arr_xx)
			   :initial-element 0.0))
	 (arr_hg_xy   (make-array (array-dimensions arr_xx)
			   :element-type (array-element-type arr_xx)
			   :initial-element 0.0))
	 (arr_hg_yy  (make-array (array-dimensions arr_xx)
			   :element-type (array-element-type arr_xx)
			   :initial-element 0.0))
	 (foo   (vigra_hourglassfilter_c arr_xx arr_xy arr_yy arr_hg_xx arr_hg_xy arr_hg_yy width height sigma rho)))
    (case foo
      ((0) (list arr_hg_xx arr_hg_xy arr_hg_yy))
      ((1) "Error in vigracl.filters.hourglassfilter: Hourglass-Filtering  of the Tensor failed!"))))

(defun hourglassfilter (tensor_image sigma rho)
  (pivot-list (mapcar #'(lambda (tensor)  (hourglassfilter-band tensor sigma rho)) (pivot-list tensor_image))))

;###############################################################################
;###################          Gaussian Sharpening           ####################

(ff:def-foreign-call vigra_gaussiansharpening_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (sharpening_factor   :float)
     (scale   :float))
  :returning  (:int fixnum))

(defun gsharpening-band (arr sharpening_factor scale)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_gaussiansharpening_c arr arr2 width height sharpening_factor scale)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.gsharpening: Gaussian sharpening failed!"))))

(defun gsharpening (image sharpening_factor scale)
  (mapcar #'(lambda (arr) (gsharpening-band arr sharpening_factor scale)) image))


;###############################################################################
;###################            Simple Sharpening           ####################

(ff:def-foreign-call vigra_simplesharpening_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (sharpening_factor   :float))
  :returning  (:int fixnum))

(defun sharpening-band (arr sharpening_factor )
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			  :initial-element 0.0))
	 (foo   (vigra_simplesharpening_c arr arr2 width height sharpening_factor)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.sharpening: Simple sharpening failed!"))))

(defun sharpening (image sharpening_factor )
  (mapcar #'(lambda (arr) (sharpening-band arr sharpening_factor )) image))
    

;###############################################################################
;###################          Nonlinear Diffusion           ####################

(ff:def-foreign-call vigra_nonlineardiffusion_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (edge_threshold   :float)
     (scale   :float))
  :returning  (:int fixnum))

(defun nonlineardiffusion-band (arr edge_threshold scale)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_nonlineardiffusion_c arr arr2 width height edge_threshold scale)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.nonlineardiffusion: Non linear Diffusion failed!!"))))

(defun nonlineardiffusion (image edge_threshold scale)
  (mapcar #'(lambda (arr) (nonlineardiffusion-band arr edge_threshold scale)) image))
    
    
;###############################################################################
;###################          Distance Transform            ####################

(ff:def-foreign-call vigra_distancetransform_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (background_label   :float)
     (norm   :int))
  :returning  (:int fixnum))

(defun distancetransform-band (arr background_label norm)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_distancetransform_c arr arr2 width height background_label norm)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.distancetransform: Distance transformation failed!!")
      ((2) "Error in vigracl.filters.distancetransform: Norm must be in {0,1,2} !!"))))

(defun distancetransform (image background_label norm)
  (mapcar #'(lambda (arr) (distancetransform-band arr background_label norm)) image))