;###############################################################################
;############ Getting Dimensions and # of Bands of an image-file    ############

(ff:def-foreign-call get_width_c 
    ((filename (* :char) string))
  :returning  (:int fixnum)
  :STRINGS-CONVERT t )

(ff:def-foreign-call get_height_c 
    ((filename (* :char) string))
  :returning  (:int fixnum)
  :STRINGS-CONVERT t )

(ff:def-foreign-call get_numbands_c 
    ((filename (* :char) string))
  :returning  (:int fixnum)
  :STRINGS-CONVERT t )


;###############################################################################
;###################             Loading images             ####################

;###############                 Grayscale images                  #############
(ff:def-foreign-call vigra_importgrayimage_c
    ((arr      (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (filename (* :char) string))
  :returning  (:int fixnum)
  :STRINGS-CONVERT t )

(defun loadgrayimage (filename)
  (let* ((width  (get_width_c filename))
	 (height (get_height_c filename))
	 (arr    (make-array (list width height) 
			     :element-type '(single-float)
			     :initial-element 0.0))
	 (foo    (vigra_importgrayimage_c arr width height filename)))
    (case foo
      ((0) (list arr))
      ((1) "Error in vigracl.impex.loadgrayimage: Image cannot be loaded by vigra!")
      ((2) "Error in vigracl.impex.loadgrayimage: Image is not grayscale!")
      ((3) "Error in vigracl.impex.loadgrayimage: Sizes do not match!"))))

;###############                  Color images                     #############
(ff:def-foreign-call vigra_importrgbimage_c
    ((arr_r      (:array :float))
     (arr_g      (:array :float))
     (arr_b      (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (filename (* :char) string))
  :returning  (:int fixnum)
  :STRINGS-CONVERT t )

(defun loadrgbimage (filename)
  (let* ((width  (get_width_c filename))
	 (height (get_height_c filename))
	 (arr_r    (make-array (list width height) 
			     :element-type '(single-float)
			     :initial-element 0.0))
	 (arr_g    (make-array (list width height) 
			     :element-type '(single-float)
			     :initial-element 0.0))
	 (arr_b    (make-array (list width height) 
			     :element-type '(single-float)
			     :initial-element 0.0))
	 (foo    (vigra_importrgbimage_c arr_r arr_g arr_b width height filename)))
    (case foo
      ((0) (list arr_r arr_g arr_b))
      ((1) "Error in vigracl.impex.loadrgbimage: Image cannot be loaded by vigra!")
      ((2) "Error in vigracl.impex.loadrgbimage: Image is not of rgb color!")
      ((3) "Error in vigracl.impex.loadrgbimage: Sizes do not match!"))))

;######    Generic (choose automatically if image is gray or colored)    #######

(defun loadimage (filename)
  (case (get_numbands_c filename)
    ((1) (loadgrayimage filename))
    ((3) (loadrgbimage filename))
    (else "Error in vigracl.impex.loadimage: Image has neither 1 nor 3 bands and thus cannot be loaded!")))

;###############################################################################
;###################             Saving images              ####################

;###############                 Grayscale images                  #############
(ff:def-foreign-call vigra_exportgrayimage_c
    ((arr      (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (filename (* :char) string))
  :returning  (:int fixnum)
  :STRINGS-CONVERT t )

(defun savegrayimage (image filename)
  (let* ((arr  (first image))
	 (width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (foo   (vigra_exportgrayimage_c arr width height filename)))
    (case foo
      ((0) t)
      ((1) "Error in vigracl.impex.savegrayimage: Image cannot be saved by vigra!"))))


;###############                RGB-Color images                  #############
(ff:def-foreign-call vigra_exportrgbimage_c
    ((arr_r      (:array :float))
     (arr_g      (:array :float))
     (arr_b      (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (filename (* :char) string))
  :returning  (:int fixnum)
  :STRINGS-CONVERT t )

(defun savergbimage (image filename)
  (let* ((arr_r (first image))
	 (arr_g (second image))
	 (arr_b (third image))
	 (width  (array-dimension arr_r 0))
	 (height (array-dimension arr_r 1))
	 (foo   (vigra_exportrgbimage_c arr_r arr_g arr_b width height filename)))
    (case foo
      ((0) t)
      ((1) "Error in vigracl.impex.savegrayimage: Image cannot be saved by vigra!"))))

;######    Generic (choose automatically if image is gray or colored)    #######

(defun saveimage (image filename)
  (case (length image)
    ((1) (savegrayimage image filename))
    ((3) (savergbimage  image filename))
    (else "Error in vigraplt.impex.saveimage: Image has neither 1 nor 3 bands and thus cannot be saved!")))
