;###############################################################################
;###################         Label image                    ####################

(ff:def-foreign-call vigra_labelimage_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum))
  :returning  (:int fixnum))

(defun labelimage-band (arr)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_labelimage_c arr arr2 width height)))
    (if (eql foo -1)
		"Error in vigracl.segmentation.labelimage: Labeling of image failed!"
      arr2)))

(defun labelimage (image)
  (mapcar #'labelimage-band image))


;###############################################################################
;###################      Watershed Transform (Union-Find)  ####################

(ff:def-foreign-call vigra_watersheds_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum))
  :returning  (:int fixnum))

(defun watersheds-band (arr)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_watersheds_c arr arr2 width height)))
    (if (eql foo -1)
		"Error in vigracl.segmentation.watersheds: Watershed Transform of image failed!"
		arr2)))
	  
(defun watersheds (image)
  (mapcar #'watersheds-band image))


;###############################################################################
;###################      Canny Edge-Detection              ####################

(ff:def-foreign-call vigra_cannyedgeimage_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (scale    :float)
     (gradient_threshold   	:float)
     (mark     :float))
  :returning  (:int fixnum))

(defun cannyedgeimage-band (arr scale gradient_threshold mark)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_cannyedgeimage_c arr arr2 width height scale gradient_threshold mark)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.segmentation:cannyedgeimage: Canny Edge Detection of image failed!"))))

(defun cannyedgeimage (image scale gradient_threshold mark)
  (mapcar #'(lambda (arr) (cannyedgeimage-band arr scale gradient_threshold mark)) image))	  


;###############################################################################
;################    Difference of Exponential Edge-Detection  #################

(ff:def-foreign-call vigra_differenceofexponentialedgeimage_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (scale   				:float)
     (gradient_threshold   	:float)
     (mark   				:float))
  :returning  (:int fixnum))

(defun differenceofexponentialedgeimage-band (arr scale gradient_threshold mark)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_differenceofexponentialedgeimage_c arr arr2 width height scale gradient_threshold mark)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.segmentation:differenceofexponentialedgeimage: Difference of Exponential Edge Detection of image failed!"))))

(defun differenceofexponentialedgeimage (image scale gradient_threshold mark)
  (mapcar #'(lambda (arr) (differenceofexponentialedgeimage-band arr scale gradient_threshold mark)) image))

		  
;###############################################################################
;###################     RegionImage -> CrackEdgeImage      ####################

(ff:def-foreign-call vigra_regionimagetocrackedgeimage_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (mark     :float))
  :returning  (:int fixnum))

(defun regionimagetocrackedgeimage-band (arr mark)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (list (- (* 2 width) 1) (- (* 2 height) 1))
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_regionimagetocrackedgeimage_c arr arr2 width height mark)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.segmentation:regionimagetocrackedgeimage: Creation of CrackEdgeImage failed!"))))

(defun regionimagetocrackedgeimage (image mark)
(mapcar #'(lambda (arr) (regionimagetocrackedgeimage-band arr mark)) image))